
pro pq2rss,p,q,erot,exscl,eyscl,exshft,eyshft,enrss,nx1,ny1,center=center, $
           quiet=quiet,anchor=anchor,rotfirst=rotfirst

;+
;NAME:
;     PQ2RSS
;PURPOSE:
;     Converts the P and Q variables in a call to poly_2d into a rotation, 
;     scale change, and shift.
;CATEGORY:
;CALLING SEQUENCE:
;     pq2rss,p,q,erot,exscl,eyscl,exshft,eyshft,enrss,nx,ny
;INPUTS:
;     p,q = p,q for poly_2d.  P and Q should be 4-element vectors and represent
;           the first order transformation from poly_warp.
;OPTIONAL INPUT PARAMETERS:
;     nx,ny = the size of the image which is being warped.  These are required
;             to fix the center of the rotation.  If /center is set, they are
;             required.
;KEYWORD PARAMETERS
;      anchor = [x0,y0] defines the center of the rotation.
;     /center = rotate about the center rather thatn the lower left corner
;               This affects only the output value of exshft and eyshft.
;     /rotfirst = Do the rotation before the shift.  Not normally a good idea.
;                 Ignored unless center or anchor is set.
;     /quiet = don't print the answer
;OUTPUTS:
;     erot = equivalent rotaion in degrees
;     exscl = equivalent x scale as a fraction of the original sacle
;     eyscl = equivalent y scale as a fraction of the original sacle
;     exshft = equivalent x shift in pixels
;     eyshft = equivalent y shift in pixels
;     enrss = warping ... this is whatever bit of p,q does not fit into the
;             simple rotation/scale/shift.  fltarr(4,2)
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;     The first order P,Q transformation also includes a warping (x*y term)
;     which is not really included here.  enrss returns the leftover P,Q
;     which represent this term to some extent.
;PROCEDURE:
;EXAMPLES:
;     IDL> pq2rss,[1,0,1,0],[1,1,0,0]
; 
;        rotation =  0.00000 degrees
;        scale x =  1.00000
;        scale y =  1.00000
;        shift x =  -1.00000
;        shift y =  -1.00000
;MODIFICATION HISTORY:
;     Written by T. Metcalf  March 30, 1993
;     TRM 27-June-93   Sepearted this code into a separate program from 
;                      align_img.pro
;     TRM 1994-11-10   Added anchor keyword.
;     TRM 2001-09-26   Added rotfirst keyword so that the default call to
;                      pq2rss is the inverse of the default call to rss2pq.
;-

   nanchor = n_elements(anchor)
   tsp = fltarr(4,2)
   tsp(*,0) = p
   tsp(*,1) = q
   erot = atan(tsp(1,0),tsp(2,0))
   erot= (erot+arange(atan(-tsp(2,1),tsp(1,1)),erot))/2.0
   ct = cos(erot)
   st = sin(erot)
   determ = tsp(2,0)*tsp(1,1) - tsp(1,0)*tsp(2,1)
   exscl = sqrt(determ*tsp(2,0)/tsp(1,1))
   eyscl = determ/exscl
   ;exscl = (tsp(1,0)/st + tsp(2,0)/ct)/2.0
   ;eyscl = (tsp(1,1)/ct - tsp(2,1)/st)/2.0
   exscl = 1./exscl
   eyscl = 1./eyscl
   if keyword_set(center) OR nanchor EQ 2 then begin
      if nanchor EQ 2 then begin
         nx12 = anchor(0)
         ny12 = anchor(1)
      endif else begin
         nx12 = nx1/2.
         ny12 = ny1/2.
      endelse
      exshft = -tsp(0,0)
      eyshft = -tsp(0,1)
      for irepeat=0,1 do begin  ; Iterate once to correct shift
         if NOT keyword_set(rotfirst) then begin
            nx12_shift = nx12 + exshft  ; Shift first is the default
            ny12_shift = ny12 + eyshft
         endif else begin
            nx12_shift = nx12           ; Rot first is not the normal case
            ny12_shift = ny12
         endelse 
         dx =  ct*nx12_shift+st*ny12_shift-nx12_shift
         dy = -st*nx12_shift+ct*ny12_shift-ny12_shift
         exshft = -tsp(0,0)-dx
         eyshft = -tsp(0,1)-dy
      endfor
   endif $
   else begin
      exshft = -tsp(0,0)
      eyshft = -tsp(0,1)
      ;x0 = nx1/2.
      ;y0 = ny1/2.
      ;zshift0 = pq2pp(tsp,x=x0,y=y0)
      ;exshft = -(zshift0(0,1)-x0)
      ;eyshft = -(zshift0(1,1)-y0)
   endelse

   ;if keyword_set(center) OR nanchor EQ 2 then begin
   ;   if nanchor EQ 2 then begin
   ;      nx12 = anchor(0)
   ;      ny12 = anchor(1)
   ;   endif else begin
   ;      nx12 = nx1/2.
   ;      ny12 = ny1/2.
   ;   endelse
   ;   halfshft1 = [ [1.,0,-nx12], $
   ;                 [0,1,-ny12], $
   ;                 [0,0,1]]
   ;   halfshft2 = [ [1.,0,nx12], $
   ;                 [0,1,ny12], $
   ;                 [0,0,1]]
   ;      rtn = halfshft1 # [ [ct,st,0], $
   ;                       [-st,ct,0], $
   ;                       [0,0,1.]] # halfshft2 
   ;endif else $
   ;   rtn = [ [ct,st,0], $
   ;           [-st,ct,0], $
   ;           [0,0,1.]]
   ;shft = [ [1,0,-exshft], $
   ;         [0,1,-eyshft], $
   ;         [0,0,1.]]
   ;scl = [ [1./exscl, 0, 0], $
   ;        [0, 1./eyscl, 0], $
   ;        [0, 0, 1. ]]
   ;tran = (shft # rtn) # scl
   ;tran = rtn # shft # scl

   erot = erot * !radeg
   enrss = tsp - rss2pq(xshift=exshft,yshift=eyshft,xscale=exscl, $
                        yscale=eyscl,rot12=erot,center=center,nx1,ny1)

   ;enrss = tsp
   ;enrss(*,0) = tsp(*,0) - [tran(2,0)*exscl,tran(1,0),tran(0,0),0.]
   ;enrss(*,1) = tsp(*,1) - [tran(2,1)*eyscl,tran(1,1),tran(0,1),0.]

   if NOT keyword_set(quiet) then begin
      print
      print,'   rotation = '+strcompress(string(erot))+' degrees'
      print,'   scale x = '+strcompress(string(exscl))
      print,'   scale y = '+strcompress(string(eyscl))
      print,'   shift x = '+strcompress(string(exshft))
      print,'   shift y = '+strcompress(string(eyshft))
   endif
end

